@@ -1,13 +1,13 @@
-# -*- coding: utf-8 -*-
-
 import sys
+import platform
 
 from datetime import datetime, timedelta, timezone
-from toot import api, config
+from toot import api, config, __version__
 from toot.auth import login_interactive, login_browser_interactive, create_app_interactive
 from toot.exceptions import ApiError, ConsoleError
 from toot.output import (print_out, print_instance, print_account, print_acct_list,
-                         print_search_results, print_timeline, print_notifications)
+                         print_search_results, print_timeline, print_notifications,
+                         print_tag_list)
 from toot.tui.utils import parse_datetime
 from toot.utils import editor_input, multiline_input, EOF_KEY
 
@@ -39,10 +39,11 @@ def get_timeline_generator(app, user, args):
         return api.home_timeline_generator(app, user, limit=args.count)
 
 
-def timeline(app, user, args):
-    generator = get_timeline_generator(app, user, args)
+def timeline(app, user, args, generator=None):
+    if not generator:
+        generator = get_timeline_generator(app, user, args)
 
-    while(True):
+    while True:
         try:
             items = next(generator)
         except StopIteration:
@@ -168,7 +169,7 @@ def unfavourite(app, user, args):
 
 
 def reblog(app, user, args):
-    api.reblog(app, user, args.status_id)
+    api.reblog(app, user, args.status_id, visibility=args.visibility)
     print_out("<green>✓ Status reblogged</green>")
 
 
@@ -197,6 +198,10 @@ def unbookmark(app, user, args):
     print_out("<green>✓ Status unbookmarked</green>")
 
 
+def bookmarks(app, user, args):
+    timeline(app, user, args, api.bookmark_timeline_generator(app, user, limit=args.count))
+
+
 def reblogged_by(app, user, args):
     for account in api.reblogged_by(app, user, args.status_id):
         print_out("{}\n @{}".format(account['display_name'], account['acct']))
@@ -220,6 +225,12 @@ def auth(app, user, args):
     print_out("\nAuth tokens are stored in: <blue>{}</blue>".format(path))
 
 
+def env(app, user, args):
+    print_out(f"toot {__version__}")
+    print_out(f"Python {sys.version}")
+    print_out(platform.platform())
+
+
 def login_cli(app, user, args):
     app = create_app_interactive(instance=args.instance, scheme=args.scheme)
     login_interactive(app, args.email)
@@ -316,6 +327,23 @@ def followers(app, user, args):
     print_acct_list(response)
 
 
+def tags_follow(app, user, args):
+    tn = args.tag_name if not args.tag_name.startswith("#") else args.tag_name[1:]
+    api.follow_tag(app, user, tn)
+    print_out("<green>✓ You are now following #{}</green>".format(tn))
+
+
+def tags_unfollow(app, user, args):
+    tn = args.tag_name if not args.tag_name.startswith("#") else args.tag_name[1:]
+    api.unfollow_tag(app, user, tn)
+    print_out("<green>✓ You are no longer following #{}</green>".format(tn))
+
+
+def tags_followed(app, user, args):
+    response = api.followed_tags(app, user)
+    print_tag_list(response)
+
+
 def mute(app, user, args):
     account = _find_account(app, user, args.account)
     api.mute(app, user, account['id'])
