@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 import logging
 import os
 import re
@@ -8,11 +6,17 @@ import sys
 
 from argparse import ArgumentParser, FileType, ArgumentTypeError
 from collections import namedtuple
+from itertools import chain
 from toot import config, commands, CLIENT_NAME, CLIENT_WEBSITE, __version__
 from toot.exceptions import ApiError, ConsoleError
 from toot.output import print_out, print_err
 
 VISIBILITY_CHOICES = ['public', 'unlisted', 'private', 'direct']
+VISIBILITY_CHOICES_STR = ", ".join(f"'{v}'" for v in VISIBILITY_CHOICES)
+
+
+def get_default_visibility():
+    return os.getenv("TOOT_POST_VISIBILITY", "public")
 
 
 def language(value):
@@ -148,6 +152,19 @@ status_id_arg = (["status_id"], {
     "type": str,
 })
 
+visibility_arg = (["-v", "--visibility"], {
+    "type": visibility,
+    "default": get_default_visibility(),
+    "help": f"Post visibility. One of: {VISIBILITY_CHOICES_STR}. Defaults to "
+            f"'{get_default_visibility()}' which can be overridden by setting "
+            "the TOOT_POST_VISIBILITY environment variable",
+})
+
+tag_arg = (["tag_name"], {
+    "type": str,
+    "help": "tag name, e.g. Caturday, or \"#Caturday\"",
+})
+
 # Arguments for selecting a timeline (see `toot.commands.get_timeline_generator`)
 common_timeline_args = [
     (["-p", "--public"], {
@@ -174,7 +191,7 @@ common_timeline_args = [
     }),
 ]
 
-timeline_args = common_timeline_args + [
+timeline_and_bookmark_args = [
     (["-c", "--count"], {
         "type": timeline_count,
         "help": "number of toots to show per page (1-20, default 10).",
@@ -192,6 +209,8 @@ timeline_args = common_timeline_args + [
     }),
 ]
 
+timeline_args = common_timeline_args + timeline_and_bookmark_args
+
 AUTH_COMMANDS = [
     Command(
         name="login",
@@ -223,6 +242,12 @@ AUTH_COMMANDS = [
         arguments=[],
         require_auth=False,
     ),
+    Command(
+        name="env",
+        description="Print environment information for inclusion in bug reports.",
+        arguments=[],
+        require_auth=False,
+    ),
 ]
 
 TUI_COMMANDS = [
@@ -317,6 +342,12 @@ READ_COMMANDS = [
         arguments=timeline_args,
         require_auth=True,
     ),
+    Command(
+        name="bookmarks",
+        description="Show bookmarked posts",
+        arguments=timeline_and_bookmark_args,
+        require_auth=True,
+    ),
 ]
 
 POST_COMMANDS = [
@@ -340,11 +371,7 @@ POST_COMMANDS = [
                 "help": "plain-text description of the media for accessibility "
                         "purposes, one per attached media"
             }),
-            (["-v", "--visibility"], {
-                "type": visibility,
-                "default": "public",
-                "help": 'post visibility, one of: %s' % ", ".join(VISIBILITY_CHOICES),
-            }),
+            visibility_arg,
             (["-s", "--sensitive"], {
                 "action": 'store_true',
                 "default": False,
@@ -428,7 +455,7 @@ STATUS_COMMANDS = [
     Command(
         name="reblog",
         description="Reblog a status",
-        arguments=[status_id_arg],
+        arguments=[status_id_arg, visibility_arg],
         require_auth=True,
     ),
     Command(
@@ -536,25 +563,47 @@ ACCOUNTS_COMMANDS = [
     ),
 ]
 
-COMMANDS = AUTH_COMMANDS + READ_COMMANDS + TUI_COMMANDS + POST_COMMANDS + STATUS_COMMANDS + ACCOUNTS_COMMANDS
+TAG_COMMANDS = [
+    Command(
+        name="tags_followed",
+        description="List hashtags you follow",
+        arguments=[],
+        require_auth=True,
+    ),
+    Command(
+        name="tags_follow",
+        description="Follow a hashtag",
+        arguments=[tag_arg],
+        require_auth=True,
+    ),
+    Command(
+        name="tags_unfollow",
+        description="Unfollow a hashtag",
+        arguments=[tag_arg],
+        require_auth=True,
+    ),
+]
+
+COMMAND_GROUPS = [
+    ("Authentication", AUTH_COMMANDS),
+    ("TUI", TUI_COMMANDS),
+    ("Read", READ_COMMANDS),
+    ("Post", POST_COMMANDS),
+    ("Status", STATUS_COMMANDS),
+    ("Accounts", ACCOUNTS_COMMANDS),
+    ("Hashtags", TAG_COMMANDS),
+]
+
+COMMANDS = list(chain(*[commands for _, commands in COMMAND_GROUPS]))
 
 
 def print_usage():
-    max_name_len = max(len(command.name) for command in COMMANDS)
-
-    groups = [
-        ("Authentication", AUTH_COMMANDS),
-        ("TUI", TUI_COMMANDS),
-        ("Read", READ_COMMANDS),
-        ("Post", POST_COMMANDS),
-        ("Status", STATUS_COMMANDS),
-        ("Accounts", ACCOUNTS_COMMANDS),
-    ]
+    max_name_len = max(len(name) for name, _ in COMMAND_GROUPS)
 
     print_out("<green>{}</green>".format(CLIENT_NAME))
     print_out("<blue>v{}</blue>".format(__version__))
 
-    for name, cmds in groups:
+    for name, cmds in COMMAND_GROUPS:
         print_out("")
         print_out(name + ":")
 
@@ -564,7 +613,7 @@ def print_usage():
 
     print_out("")
     print_out("To get help for each command run:")
-    print_out("  <yellow>toot <command> --help</yellow>")
+    print_out("  <yellow>toot \\<command> --help</yellow>")
     print_out("")
     print_out("<green>{}</green>".format(CLIENT_WEBSITE))
 
@@ -589,8 +638,8 @@ def run_command(app, user, name, args):
     command = next((c for c in COMMANDS if c.name == name), None)
 
     if not command:
-        print_err("Unknown command '{}'\n".format(name))
-        print_usage()
+        print_err(f"Unknown command '{name}'")
+        print_out("Run <yellow>toot --help</yellow> to show a list of available commands.")
         return
 
     parser = get_argument_parser(name, command)
